#include <u.h>
#include <libc.h>
#include <auth.h>
#include <ctype.h>
#include "octopus.h"


static struct {
	int n;
	char *s;
} Errs[] = {
	/* 1xx info */
	{ 100, "continue sending" },
	{ 102, "in progress" },

	/* 2xx Success */
	{ 200, "ok" },
	{ 201, "created" },
	{ 202, "accepted" },
	{ 203, "not authoritative information" },
	{ 204, "no content" },
	{ 205, "reset content" },		// client should re-request data ?
	{ 206, "partial content" },		// reply to GET with Range: spec
	{ 207, "multi status" },

	/* 3xx Redirection */
	{ 300, "multiple choices" },
	{ 301, "moved permanently" },
	{ 302, "found - moved temporarily" },
	{ 303, "see other - redirected by script" },
	{ 304, "not modified" },
	{ 305, "use proxy" },
	{ 306, "switch proxy - depricated" },
	{ 307, "temporary redirect" },

	/* 4xx Client Error */
	{ 400, "bad request" },
	{ 401, "authentication failed" },
	{ 402, "payment required" },
	{ 403, "forbidden - irresepective of auth" },
	{ 404, "does not exist" },
	{ 405, "method not allowed" },
	{ 406, "not acceptable" },
	{ 407, "proxy authentication required" },
	{ 408, "request timeout" },
	{ 409, "conflict" },
	{ 410, "gone - temporarly unavailable" },
	{ 411, "length required" },
	{ 412, "precondition failed" },
	{ 413, "request too big" },
	{ 414, "request uri too long" },
	{ 415, "unsupported media type" },
	{ 416, "requested range not satisfiable" },
	{ 417, "expectation failed" },
	{ 418, "i'm a teapot - April fool" },
	{ 421, "connections from your ip" },
	{ 422, "request not well-formed" },
	{ 423, "locked" },
	{ 424, "failed dependency" },
	{ 425, "unordered collection" },
	{ 426, "upgrade required - (use TLS?)" },
	{ 449, "retry with" },
	{ 450, "blocked by parental controls" },

	/* 5xx Server Error */
	{ 500, "internal server error" },
	{ 501, "not implemented" },
	{ 502, "bad gateway" },
	{ 503, "service unavailable" },
	{ 504, "gateway timeout" },
	{ 505, "http version not supported" },
	{ 506, "variant also negotiates - circular references" },
	{ 507, "insufficient storage" },
	{ 509, "bandwidth limit exceeded - Apache special" },
	{ 510, "not extended" },
	{ 530, "user access denied" },
};

void
httperrstr(int n)
{
	int i;

	for(i = 0; i < nelem(Errs); i++)
		if(Errs[i].n == n)
			break;
	if(i < nelem(Errs))
		werrstr("%s", Errs[i].s);
	else
		werrstr("%d: unknown error", n);
}

int
exch(Sess *s, char *post, char *fmt, ...)
{
	va_list ap;
	int e, fd;
	char *url, *tmp, *a[4], buf[1024], err[ERRMAX];

	va_start(ap, fmt);
	if((url = vsmprint(fmt, ap)) == nil)
		sysfatal("nil url");
	va_end(ap);

	if(Debug && strstr(Debug, "url"))
		fprint(2, "request: %s/%q\n", s->url, url);
	/* set headers */
	snprint(buf, sizeof buf, "%s/%d/ctl", s->webfs, s->conid);
	fd = open(buf, OWRITE);
	if(fd == -1)
		sysfatal("%s cannot open, %r", buf);

	if(post)
		fprint(fd, "request POST");

	tmp = smprint("url %s/%s", s->url, url);
	write(fd, tmp, strlen(tmp));
	free(tmp);
	close(fd);

	if(post){
		/* send post body */
		if(Debug && strstr(Debug, "url"))
			fprint(2, "post: %s\n", post);

		snprint(buf, sizeof buf, "%s/%d/postbody", s->webfs, s->conid);
		fd = open(buf, OWRITE);
		if(fd == -1)
			sysfatal("%s open failed, %r", buf);
		fprint(fd, "%s", post);
		close(fd);
	}
	free(url);

	/* read reply */
	snprint(buf, sizeof buf, "%s/%d/body", s->webfs, s->conid);
	fd = open(buf, OREAD);
	if(fd < 0){
		rerrstr(err, sizeof(err));

		if(tokenize(err, a, nelem(a)) >= 2)
			if((e = atoi(a[1])) > 0)
				httperrstr(e);
		return -1;
	}

	return fd;
}

int
dialweb(Sess *s, char *webfs, char *url)
{
	int n, fd;
	char buf[128];

	snprint(buf, sizeof(buf), "%s/clone", webfs);
	if((fd = open(buf, ORDWR)) < 0)
		return -1;
	if((n = read(fd, buf, sizeof buf-1)) <= 0){
		close(fd);
		return -1;
	}
	buf[n] = '\0';
	if((s->conid = atoi(buf)) <0){
		close(fd);
		return -1;
	}
	s->fd = fd;
	s->url = url;
	s->webfs = webfs;
	return 0;
}

int
auth(Sess *s, char *server)
{
	int fd;
	char buf[255];

	UserPasswd *up;
	if((up = auth_getuserpasswd(auth_getkey, "server=%s proto=pass service=octopus", server)) == 0)
		return -1;

	srand(nsec());
	s->nonce = lrand();

	snprint(buf, sizeof(buf), "Username=%s&Password=%s&rndval=%lud", up->user, up->passwd, s->nonce);
	if((fd = exch(s, buf, "portal/login.asp?ajax=t")) == -1)
		return -1;
	memset(buf, strlen(buf), 0xff);

	close(fd);
	return 0;
}

char *
trimdup(char *s)
{
	char *t;

	while(isspace(*s))
		s++;
	for(t = strchr(s, 0) -1; isspace(*t) && t >= s; t--)
		*t = 0;

	if((t = strdup(s)) == nil)
		sysfatal("trimdup() failed, %r");
	return t;
}

char *
trim(char *s)
{
	char *t;

	while(isspace(*s))
		s++;
	for(t = strchr(s, 0) -1; isspace(*t) && t >= s; t--)
		*t = 0;
	return s;
}

char *
dupstr(char *s)
{
	char *t, *u;

	if(s == nil)
		return nil;

	while(isspace(*s))
		s++;
	if((t = strdup(s)) == nil)
		sysfatal("dupstr() failed, %r");
	for(u = strchr(t, 0) -1; isspace(*u) && u >= t; u--)
		*u = 0;
	return t;
}

char *
strlwr(char *s)
{
	char *t;

	for(t = s; *t; t++)
		*t = tolower(*t);
	return s;
}
