#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "octopus.h"

typedef struct User User;
struct User {
	char *id;				/* internal ID */
	char *number;			/* employee number */
	char *surname;
	char *forename;
	char *phone;
	char *photo;			/* photo ID or zero if none */
	char *email;
	char *jobtype;
	char *country;
	char *dept;
	char *team;
	char *costcenter;
	char *manager;
	User *left;
	User *right;
};

char *Debug;
char *Site;
int Verbose;
int Dump;

User *
newuser(User *u, char *number, char *forename, char *surname, char *email, char *phone, char *photo, char *id)
{
	int n;

	if(u == nil){
		u = mallocz(sizeof(User), 1);
		if(u == nil)
			sysfatal("No memory, %r\n");
		u->number = dupstr(number);
		u->forename = dupstr(forename);
		u->surname = dupstr(surname);
		u->email = dupstr(email);
		u->phone = dupstr(phone);
		u->photo = dupstr(photo);
		u->id = dupstr(id);
		u->left = u->right = nil;
		return u;
	}

	n = strcmp(u->number, number);

	if(n == 0)
		fprint(2, "%s %s == %s %s, users have the same employee number\n",
			u->forename, u->surname, forename, surname);

	if(n < 0)
		u->left = newuser(u->left, number, forename, surname, email, phone, photo, id);
	else
		u->right = newuser(u->right, number, forename, surname, email, phone, photo, id);
	return u;
}

void
dumpusers(Biobuf *bp, User *u)
{
	if(u == nil)
		return;

	dumpusers(bp, u->left);
	Bprint(bp, "%s:", (u->forename)? u->forename: "");
	Bprint(bp, "%s:", (u->surname)? u->surname: "");
	Bprint(bp, "%s:", (u->number)? u->number: "");
//	Bprint(bp, "%s:", (u->id)? u->id: "");
	Bprint(bp, "%s:", (u->photo)? u->photo: "");
	Bprint(bp, "%s:", (u->email)? u->email: "");
	Bprint(bp, "%s:", (u->phone)? u->phone: "");
	Bprint(bp, "%s:", (u->jobtype)? u->jobtype: "");
	Bprint(bp, "%s:", (u->country)? u->country: "");
	Bprint(bp, "%s:", (u->dept)? u->dept: "");
	Bprint(bp, "%s:", (u->team)? u->team: "");
	Bprint(bp, "%s:", (u->costcenter)? u->costcenter: "");
	Bprint(bp, "%s:", (u->manager)? u->manager: "");
	Bprint(bp, "\n");
	dumpusers(bp, u->right);
}

User *
search_results(User *u, Xml *xml)
{
	Elem *base, *ep, *x;
	char *id, *number, *email, *surname, *forename, *phone, *photo;

	if((base = xml_find(xml, xml->root, "OutputEmployeeAsXml/Employee")) == nil)
		sysfatal("<OutputEmployeeAsXml> <Employee> not found, unexpected search format");

	for(ep = base; ep; ep = ep->next){
		number = email = surname = forename = phone = photo = id = nil;

		if((x = xml_look(ep->child, "EmployeeNumber", nil, nil)) != nil)
			number = x->pcdata;
		if((x = xml_look(ep->child, "EmailWork", nil, nil)) != nil && x->pcdata)
			email = strlwr(x->pcdata);
		if((x = xml_look(ep->child, "Surname", nil, nil)) != nil)
			surname = x->pcdata;
		if((x = xml_look(ep->child, "KnownAs", nil, nil)) != nil)
			forename = x->pcdata;
		if((x = xml_look(ep->child, "PhoneWork", nil, nil)) != nil)
			phone = x->pcdata;
		if((x = xml_look(ep->child, "PhotoID", nil, nil)) != nil){
			photo = x->pcdata;
			if(photo && atoi(photo) == 0)
				photo = nil;
		}
		if((x = xml_look(ep->child, "ID", nil, nil)) != nil)
			id = x->pcdata;

		u = newuser(u, number, forename, surname, email, phone, photo, id);
	}
	return u;
}


char *
fixmanager(char *buf)
{
	int n;
	char *name, *a[4];

	if(buf == nil)
		return nil;

	if((n = getfields(buf, a, nelem(a), 1, " (),")) != 3)
		sysfatal("fixmanager: bad arg count, wanted 3, got %d\n", n);

	if((name = smprint("%s %s", a[1], a[0])) == nil)
		sysfatal("fixmanager: No memory, %r\n");
	return name;
}
		
int
detail_results(Xml *xml, User *u)
{
	Elem *ep, *right, *left;
	char *name, *value, *number, *email, *jobtype, *country, *dept, *team, *costcenter, *manager;

	number = email = jobtype = country = dept = team = costcenter = manager = nil;

	for(ep = xml->root; (left = xml_look(ep, "div", "class", "dirDetL")) != nil; ep = right->next){
		name = trim(left->pcdata);

		value = nil;
		right = xml_look(left, "div", "class", "dirDetR");
		if(right && right->pcdata)
			value = trim(right->pcdata);
		else
		if(right->child && right->child->pcdata)
			value = trim(right->child->pcdata);

		if(strcmp(name, "Employee Number") == 0)
			number = value;
		if(strcmp(name, "Work Email") == 0 && value)
			email = strlwr(value);
		if(strcmp(name, "Job Type") == 0)
			jobtype = value;
		if(strcmp(name, "Country") == 0)
			country = value;
		if(strcmp(name, "Department") == 0)
			dept = value;
		if(strcmp(name, "Team") == 0)
			team = value;
		if(strcmp(name, "Cost Centre") == 0)
			costcenter = value;
		if(strcmp(name, "Line Manager") == 0)
			manager = value;
	}

	if(number == nil){
		fprint(2, "%s: %s %s no user detail\n", argv0, u->forename, u->surname);
		return -1;
	}

	if(strcmp(u->number, number) != 0){
		fprint(2, "%s: %s %s employee numbers differ: %s %s\n",
			argv0, u->forename, u->surname, u->number, number);
		return -1;
	}

	if(email){
		if(u->email){
			if(strcmp(u->email, email) != 0)
				fprint(2, "%s: %s %s email accounts differ: %s %s\n",
					argv0, u->forename, u->surname, u->email, email);
			free(u->email);
			u->email = dupstr(email);
		}
	}
	if(jobtype)		u->jobtype = dupstr(jobtype);
	if(country)		u->country = dupstr(country);
	if(dept)		u->dept = dupstr(dept);
	if(team)		u->team = dupstr(team);
	if(costcenter)	u->costcenter = dupstr(costcenter);
	if(manager)		u->manager = fixmanager(manager);

	return 0;
}

void
writexml(Xml *xml, char *fmt, ...)
{
	char *file;
	va_list ap;
	Biobuf *bp;

	va_start(ap, fmt);
	file = vsmprint(fmt, ap);
	va_end(ap);

	if(Verbose)
		fprint(2, "write %s\n", file);

	if((bp = Bopen(file, OWRITE)) == nil)
		sysfatal("%s cannot open, %r", file);
	xml_print(xml, 1, bp);
	free(file);
	Bterm(bp);
}

int
openfile(char *fmt, ...)
{
	int fd;
	char *file;
	va_list ap;

	va_start(ap, fmt);
	file = vsmprint(fmt, ap);
	va_end(ap);

	if(Verbose)
		fprint(2, "open %s\n", file);

	if((fd = open(file, OREAD)) < 0)
		sysfatal("%s cannot open, %r", file);
	return(fd);
}

int
details_request(Sess *s, User *u)
{
	int fd;
	Xml *xml;
	char *line;
	Biobuf net;

	if(Verbose)
		fprint(2, "get details %s, %s %s\n", u->id, u->forename, u->surname);

	if((fd = exch(s, nil, "portal/directory/detail.aspx?id=%s&rndval=%lud", u->id, s->nonce)) == -1){
		fprint(2, "%s: cannot get user detail, id=%s, %r\n", argv0, u->id);
		return -1;
	}

	if(Dump == 2){
		int n, out;
		char buf[8192];

		if((out = create("search.raw", OWRITE, 0644)) < 0)
			fprint(2, "search.raw cannot create %r\n");
		else{
			while((n = read(fd, buf, sizeof(buf))) > 0)
				write(out, buf, n);
			close(out);
		}
	}

	Binit(&net, fd, OREAD);

	/*
	 * This is Junk. The deteial comes back as badly formed HTML with an XHTML tail
	 * and we have to use this very fragile test to strip the former so we can parse the latter.
	 * This will break, I can only hope it will work for a while.
	 */
	while((line = Brdline(&net, '\n')) != nil){			/* skip non-hierarchal junk */
		line[Blinelen(&net) -1] = 0;
		if(strcmp(line, "<div class=\"cb\"><!-- --></div>\r") == 0)
			break;
	}

	xml = xml_parse(&net, 8192, Fcrushwhite);
	Bterm(&net);
	close(fd);

	if(xml == nil){
		fprint(2, "%s: detail xml, parse failed\n", argv0);
		return -1;
	}

	if(Dump == 1)
		writexml(xml, "detail.xml");

	if(detail_results(xml, u) < 0){
		fprint(2, "%s: failed to walk detail xml\n", argv0);
		return -1;
	}
	xml_free(xml);
	return 0;
}


int
get_details(Sess *s, User *u)
{
	if(u){
		if(get_details(s, u->left) < 0)
			return -1;
		if(details_request(s, u) < 0)
			return -1;
		sleep(2);
		if(get_details(s, u->right) < 0)
			return -1;
	}
	return 0;
}


void
usage(void)
{
	fprint(2, "usage: %s [-w webfs]\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	int fd;
	Xml *xml;
	Sess *s;
	User *users;
	char *webfs, *host;
	Biobuf out, net;

	Dump = 0;
	host = nil;
	users = nil;
	Debug = nil;
	webfs = "/mnt/web";
	ARGBEGIN{
	case 'd':
		Debug = EARGF(usage());
		break;
	case 'D':
		Dump++;
		break;
	case 'h':
		host = EARGF(usage());
		break;
	case 'v':
		Verbose++;
		break;
	case 'w':
		webfs = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if((Site = getenv("site")) == nil)
		sysfatal("$site environment variable not set");

	if(host == nil)
		host = getenv("octopus");
	if(host == nil)
		sysfatal("no site to connect to, set $octopus or use -h <host>t");

	if((s = mallocz(sizeof(Sess), 1)) == nil)
		sysfatal("No memory");

	if(dialweb(s, webfs, host) < 0)
		sysfatal("cannot get session to webfs");

	if(auth(s, host) < 0)
		sysfatal("authentication failed");

	if((fd = exch(s, nil, "portal/directory/search.aspx?ajax=search&t=&ViewGroup=false&id=0&count=9000&rndval=%lud", s->nonce)) == -1)
		sysfatal("cannot get user directory %r");

	Binit(&net, fd, OREAD);
	xml = xml_parse(&net, 8192, Fcrushwhite);
	Bterm(&net);
	close(fd);

	if(xml == nil)
		sysfatal("search xml, parse failed");
	close(fd);

	if(Dump == 1)
		writexml(xml, "search.xml");

	users = search_results(users, xml);
	xml_free(xml);

	if(Verbose)
		fprint(2, "search OK\n");

	get_details(s, users);

	Binit(&out, 1, OWRITE);
	dumpusers(&out, users);
	Bterm(&out);

	exits(nil);
}
