#include <u.h>
#include <libc.h>
#include <bio.h>
#include <xml.h>
#include "octopus.h"

char *Debug;
int Verbose;
int Dump;


void
usage(void)
{
	fprint(2, "usage: %s [-w webfs] [-h host] user@host.dom\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	int fd, n;
	Sess *s;
	Biobuf out;
	char buf[8192];
	char *webfs, *host;

	Dump = 0;
	host = nil;
	Debug = nil;
	webfs = "/mnt/web";
	ARGBEGIN{
	case 'd':
		Debug = EARGF(usage());
		break;
	case 'D':
		Dump++;
		break;
	case 'v':
		Verbose++;
		break;
	case 'w':
		webfs = EARGF(usage());
		break;
	case 'h':
		host = EARGF(usage());
		break;
	default:
		usage();
	}ARGEND;

	if(argc != 2)
		usage();

	if(host == nil)
		host = getenv("octopus");
	if(host == nil)
		sysfatal("no site to connect to, set $octopus or use -h <host>t");

	if((s = mallocz(sizeof(Sess), 1)) == nil)
		sysfatal("No memory");

	if(dialweb(s, webfs, host) < 0)
		sysfatal("cannot get session to webfs");

	if(auth(s, host) < 0)
		sysfatal("authentication failed");
	if((fd = exch(s, nil, "portal/my/photo.aspx?id=%s", argv[0])) == -1)
		sysfatal("cannot get user directory %r");

	Binit(&out, 1, OWRITE);
	while((n = read(fd, buf, sizeof(buf))) > 0)
		Bwrite(&out, buf, n);
	Bterm(&out);
	close(fd);

	exits(nil);
}
